@@ -238,6 +238,8 @@ void ColorConversionPipeline::init_ops()
   ops.emplace_back(std::make_shared<Op_RRGGBBxx_HDR_to_YCbCr420>());
   ops.emplace_back(std::make_shared<Op_RGB24_32_to_YCbCr444_GBR>());
   ops.emplace_back(std::make_shared<Op_drop_alpha_plane>());
+  ops.emplace_back(std::make_shared<Op_flatten_alpha_plane<uint8_t>>());
+  ops.emplace_back(std::make_shared<Op_flatten_alpha_plane<uint16_t>>());
   ops.emplace_back(std::make_shared<Op_to_hdr_planes>());
   ops.emplace_back(std::make_shared<Op_to_sdr_planes>());
   ops.emplace_back(std::make_shared<Op_YCbCr420_bilinear_to_YCbCr444<uint8_t>>());
@@ -260,11 +262,13 @@ void ColorConversionPipeline::release_ops()
 
 bool ColorConversionPipeline::construct_pipeline(const ColorState& input_state,
                                                  const ColorState& target_state,
-                                                 const heif_color_conversion_options& options)
+                                                 const heif_color_conversion_options& options,
+                                                 const heif_color_conversion_options_ext& options_ext)
 {
   m_conversion_steps.clear();
 
   m_options = options;
+  m_options_ext = options_ext;
 
   if (input_state == target_state) {
     return true;
@@ -360,7 +364,7 @@ bool ColorConversionPipeline::construct_pipeline(const ColorState& input_state,
 
       auto out_states = op_ptr->state_after_conversion(processed_states.back().output_state,
                                                        target_state,
-                                                       options);
+                                                       options, options_ext);
       for (const auto& out_state : out_states) {
         int new_op_costs = out_state.speed_costs + processed_states.back().speed_costs;
 #if DEBUG_PIPELINE_CREATION
@@ -427,7 +431,8 @@ std::string ColorConversionPipeline::debug_dump_pipeline() const
 }
 
 
-std::shared_ptr<HeifPixelImage> ColorConversionPipeline::convert_image(const std::shared_ptr<HeifPixelImage>& input)
+Result<std::shared_ptr<HeifPixelImage>> ColorConversionPipeline::convert_image(const std::shared_ptr<HeifPixelImage>& input,
+                                                                               const heif_security_limits* limits)
 {
   std::shared_ptr<HeifPixelImage> in = input;
   std::shared_ptr<HeifPixelImage> out = in;
@@ -439,9 +444,12 @@ std::shared_ptr<HeifPixelImage> ColorConversionPipeline::convert_image(const std
     print_spec(std::cerr, in);
 #endif
 
-    out = step.operation->convert_colorspace(in, step.input_state, step.output_state, m_options);
-    if (!out) {
-      return nullptr; // TODO: we should return a proper error
+    auto outResult = step.operation->convert_colorspace(in, step.input_state, step.output_state, m_options, m_options_ext, limits);
+    if (outResult.error) {
+      return outResult.error;
+    }
+    else {
+      out = *outResult;
     }
 
     // --- pass the color profiles to the new image
@@ -467,6 +475,16 @@ std::shared_ptr<HeifPixelImage> ColorConversionPipeline::convert_image(const std
       out->set_pixel_ratio(h, v);
     }
 
+    if (in->has_gimi_sample_content_id()) {
+      out->set_gimi_sample_content_id(in->get_gimi_sample_content_id());
+    }
+
+    if (auto* tai = in->get_tai_timestamp()) {
+      out->set_tai_timestamp(tai);
+    }
+
+    out->set_sample_duration(in->get_sample_duration());
+
     const auto& warnings = in->get_warnings();
     for (const auto& warning : warnings) {
       out->add_warning(warning);
@@ -479,13 +497,21 @@ std::shared_ptr<HeifPixelImage> ColorConversionPipeline::convert_image(const std
 }
 
 
-std::shared_ptr<HeifPixelImage> convert_colorspace(const std::shared_ptr<HeifPixelImage>& input,
-                                                   heif_colorspace target_colorspace,
-                                                   heif_chroma target_chroma,
-                                                   const std::shared_ptr<const color_profile_nclx>& target_profile,
-                                                   int output_bpp,
-                                                   const heif_color_conversion_options& options)
+Result<std::shared_ptr<HeifPixelImage>> convert_colorspace(const std::shared_ptr<HeifPixelImage>& input,
+                                                           heif_colorspace target_colorspace,
+                                                           heif_chroma target_chroma,
+                                                           const std::shared_ptr<const color_profile_nclx>& target_profile,
+                                                           int output_bpp,
+                                                           const heif_color_conversion_options& options,
+                                                           const heif_color_conversion_options_ext* options_ext_optional,
+                                                           const heif_security_limits* limits)
 {
+  std::unique_ptr<heif_color_conversion_options_ext, void(*)(heif_color_conversion_options_ext*)>
+      options_ext(heif_color_conversion_options_ext_alloc(), heif_color_conversion_options_ext_free);
+
+  heif_color_conversion_options_ext_copy(options_ext.get(), options_ext_optional);
+
+
   // --- check that input image is valid
 
   uint32_t width = input->get_width();
@@ -496,7 +522,7 @@ std::shared_ptr<HeifPixelImage> convert_colorspace(const std::shared_ptr<HeifPix
   if (input->has_channel(heif_channel_Alpha)) {
     if (input->get_width(heif_channel_Alpha) != width ||
         input->get_height(heif_channel_Alpha) != height) {
-      return nullptr;
+      return Error::InternalError;
     }
   }
 
@@ -506,7 +532,7 @@ std::shared_ptr<HeifPixelImage> convert_colorspace(const std::shared_ptr<HeifPix
     if (target_chroma != heif_chroma_420 &&
         target_chroma != heif_chroma_422 &&
         target_chroma != heif_chroma_444) {
-      return nullptr;
+      return Error::InternalError;
     }
   }
 
@@ -555,7 +581,12 @@ std::shared_ptr<HeifPixelImage> convert_colorspace(const std::shared_ptr<HeifPix
     output_state.has_alpha = is_interleaved_with_alpha(target_chroma);
   }
   else {
-    output_state.has_alpha = input_state.has_alpha;
+    if (options_ext->alpha_composition_mode != heif_alpha_composition_mode_none) {
+      output_state.has_alpha = false;
+    }
+    else {
+      output_state.has_alpha = input_state.has_alpha;
+    }
   }
 
   if (output_bpp) {
@@ -582,28 +613,39 @@ std::shared_ptr<HeifPixelImage> convert_colorspace(const std::shared_ptr<HeifPix
   }
 
   ColorConversionPipeline pipeline;
-  bool success = pipeline.construct_pipeline(input_state, output_state, options);
+  bool success = pipeline.construct_pipeline(input_state, output_state, options, *options_ext);
   if (!success) {
-    return nullptr;
+    return Error{heif_error_Unsupported_feature,
+                 heif_suberror_Unsupported_color_conversion};
   }
 
   if (pipeline.is_nop()) {
     return input;
   }
   else {
-    return pipeline.convert_image(input);
+    return pipeline.convert_image(input, limits);
   }
 }
 
 
-std::shared_ptr<const HeifPixelImage> convert_colorspace(const std::shared_ptr<const HeifPixelImage>& input,
-                                                         heif_colorspace colorspace,
-                                                         heif_chroma chroma,
-                                                         const std::shared_ptr<const color_profile_nclx>& target_profile,
-                                                         int output_bpp,
-                                                         const heif_color_conversion_options& options)
+Result<std::shared_ptr<const HeifPixelImage>> convert_colorspace(const std::shared_ptr<const HeifPixelImage>& input,
+                                                                 heif_colorspace colorspace,
+                                                                 heif_chroma chroma,
+                                                                 const std::shared_ptr<const color_profile_nclx>& target_profile,
+                                                                 int output_bpp,
+                                                                 const heif_color_conversion_options& options,
+                                                                 const heif_color_conversion_options_ext* options_ext,
+                                                                 const heif_security_limits* limits)
 {
   std::shared_ptr<HeifPixelImage> non_const_input = std::const_pointer_cast<HeifPixelImage>(input);
 
-  return convert_colorspace(non_const_input, colorspace, chroma, target_profile, output_bpp, options);
+  auto result = convert_colorspace(non_const_input, colorspace, chroma, target_profile, output_bpp, options, options_ext, limits);
+  if (result.error) {
+    return result.error;
+  }
+  else {
+    // TODO: can we simplify this? It's a bit awkward to do these assignments just to get a "const HeifPixelImage".
+    std::shared_ptr<const HeifPixelImage> constImage = *result;
+    return constImage;
+  }
 }
