@@ -81,7 +81,7 @@ static void rav1e_set_default_parameters(void* encoder);
 
 static const char* rav1e_plugin_name()
 {
-  strcpy(plugin_name, "Rav1e encoder");
+  snprintf(plugin_name, MAX_PLUGIN_NAME_LENGTH, "Rav1e encoder v%s", rav1e_version_short());
 
   return plugin_name;
 }
@@ -415,19 +415,22 @@ struct heif_error rav1e_get_parameter_string(void* encoder_raw, const char* name
 {
   struct encoder_struct_rav1e* encoder = (struct encoder_struct_rav1e*) encoder_raw;
 
-  switch (encoder->chroma) {
-    case heif_chroma_420:
-      save_strcpy(value, value_size, "420");
-      break;
-    case heif_chroma_422:
-      save_strcpy(value, value_size, "422");
-      break;
-    case heif_chroma_444:
-      save_strcpy(value, value_size, "444");
-      break;
-    default:
-      assert(false);
-      return heif_error_invalid_parameter_value;
+  if (strcmp(name, kParam_chroma) == 0) {
+    switch (encoder->chroma) {
+      case heif_chroma_420:
+        save_strcpy(value, value_size, "420");
+        break;
+      case heif_chroma_422:
+        save_strcpy(value, value_size, "422");
+        break;
+      case heif_chroma_444:
+        save_strcpy(value, value_size, "444");
+        break;
+      default:
+        assert(false);
+        return heif_error_invalid_parameter_value;
+    }
+    return heif_error_ok;
   }
 
   return heif_error_unsupported_parameter;
@@ -605,12 +608,12 @@ struct heif_error rav1e_encode_image(void* encoder_raw, const struct heif_image*
   // if (input_class == heif_image_input_class_alpha) {
   //} else
   {
-    int strideY;
-    const uint8_t* Y = heif_image_get_plane_readonly(image, heif_channel_Y, &strideY);
-    int strideCb;
-    const uint8_t* Cb = heif_image_get_plane_readonly(image, heif_channel_Cb, &strideCb);
-    int strideCr;
-    const uint8_t* Cr = heif_image_get_plane_readonly(image, heif_channel_Cr, &strideCr);
+    size_t strideY;
+    const uint8_t* Y = heif_image_get_plane_readonly2(image, heif_channel_Y, &strideY);
+    size_t strideCb;
+    const uint8_t* Cb = heif_image_get_plane_readonly2(image, heif_channel_Cb, &strideCb);
+    size_t strideCr;
+    const uint8_t* Cr = heif_image_get_plane_readonly2(image, heif_channel_Cr, &strideCr);
 
 
     uint32_t height = heif_image_get_height(image, heif_channel_Y);
